"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const rpc_1 = require("../../rpc");
const artifacts_1 = require("./lib/artifacts");
const config_1 = require("./lib/config");
const dependencies_1 = require("./lib/dependencies");
const diagnostics_1 = require("./lib/diagnostics");
const performance_1 = require("./lib/performance");
const program_1 = require("./lib/program/program");
const solution_builder_1 = require("./lib/program/solution-builder");
const watch_program_1 = require("./lib/program/watch-program");
const system_1 = require("./lib/system");
const tracing_1 = require("./lib/tracing");
const tsbuildinfo_1 = require("./lib/tsbuildinfo");
const worker_config_1 = require("./lib/worker-config");
const getIssuesWorker = ({ changedFiles = [], deletedFiles = [] }, watching) => __awaiter(void 0, void 0, void 0, function* () {
    system_1.system.invalidateCache();
    if (config_1.didConfigFileChanged({ changedFiles, deletedFiles })) {
        config_1.invalidateConfig();
        dependencies_1.invalidateDependencies();
        artifacts_1.invalidateArtifacts();
        diagnostics_1.invalidateDiagnostics();
        program_1.invalidateProgram(true);
        watch_program_1.invalidateWatchProgram(true);
        solution_builder_1.invalidateSolutionBuilder(true);
        tsbuildinfo_1.invalidateTsBuildInfo();
    }
    else if (config_1.didRootFilesChanged()) {
        dependencies_1.invalidateDependencies();
        artifacts_1.invalidateArtifacts();
        watch_program_1.invalidateWatchProgramRootFileNames();
        solution_builder_1.invalidateSolutionBuilder();
    }
    artifacts_1.registerArtifacts();
    performance_1.enablePerformanceIfNeeded();
    const parseConfigIssues = config_1.getParseConfigIssues();
    if (parseConfigIssues.length) {
        // report config parse issues and exit
        return parseConfigIssues;
    }
    // use proper implementation based on the config
    if (worker_config_1.config.build) {
        solution_builder_1.useSolutionBuilder();
    }
    else if (watching) {
        watch_program_1.useWatchProgram();
    }
    else {
        program_1.useProgram();
    }
    // simulate file system events
    changedFiles.forEach((changedFile) => {
        system_1.system === null || system_1.system === void 0 ? void 0 : system_1.system.invokeFileChanged(changedFile);
    });
    deletedFiles.forEach((removedFile) => {
        system_1.system === null || system_1.system === void 0 ? void 0 : system_1.system.invokeFileDeleted(removedFile);
    });
    // wait for all queued events to be processed
    yield system_1.system.waitForQueued();
    // retrieve all collected diagnostics as normalized issues
    const issues = diagnostics_1.getIssues();
    tracing_1.dumpTracingLegendIfNeeded();
    performance_1.printPerformanceMeasuresIfNeeded();
    performance_1.disablePerformanceIfNeeded();
    return issues;
});
rpc_1.exposeRpc(getIssuesWorker);
