"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.invalidateProgram = exports.useProgram = void 0;
const config_1 = require("../config");
const diagnostics_1 = require("../diagnostics");
const compiler_host_1 = require("../host/compiler-host");
const typescript_1 = require("../typescript");
let compilerHost;
let program;
function useProgram() {
    const parsedConfig = config_1.getParsedConfig();
    if (!compilerHost) {
        compilerHost = compiler_host_1.createCompilerHost(parsedConfig);
    }
    if (!program) {
        program = typescript_1.typescript.createProgram({
            rootNames: parsedConfig.fileNames,
            options: parsedConfig.options,
            projectReferences: parsedConfig.projectReferences,
            host: compilerHost,
        });
    }
    diagnostics_1.updateDiagnostics(config_1.getConfigFilePathFromProgram(program), diagnostics_1.getDiagnosticsOfProgram(program));
}
exports.useProgram = useProgram;
function invalidateProgram(withHost = false) {
    if (withHost) {
        compilerHost = undefined;
    }
    program = undefined;
}
exports.invalidateProgram = invalidateProgram;
